package db

import (
	"encoding/json"

	bolt "go.etcd.io/bbolt"
	"golang.org/x/xerrors"

	"github.com/aquasecurity/trivy-db/pkg/types"
)

const (
	vulnerabilityBucket = "vulnerability"
)

func (dbc Config) PutVulnerability(tx *bolt.Tx, cveID string, vuln types.Vulnerability) error {
	if err := dbc.put(tx, []string{vulnerabilityBucket}, cveID, vuln); err != nil {
		return xerrors.Errorf("failed to put severity: %w", err)
	}
	return nil
}

func (dbc Config) GetVulnerability(cveID string) (vuln types.Vulnerability, err error) {
	err = db.View(func(tx *bolt.Tx) error {
		bucket := tx.Bucket([]byte(vulnerabilityBucket))
		value := bucket.Get([]byte(cveID))
		if value == nil {
			return xerrors.Errorf("no vulnerability details for %s", cveID)
		}
		if err = json.Unmarshal(value, &vuln); err != nil {
			return xerrors.Errorf("failed to unmarshal JSON: %w", err)
		}
		return nil
	})
	if err != nil {
		return types.Vulnerability{}, xerrors.Errorf("failed to get the vulnerability %q: %w", cveID, err)
	}
	return vuln, nil
}
